@@ -375,7 +375,7 @@ int manager_get_link_dhcp4_client_identifier(const IfNameIndex *ifidx, DHCPClien
         return 0;
 }
 
-int manager_set_link_dhcp_client_iaid(const IfNameIndex *ifidx, const DHCPClient kind, const uint32_t iaid) {
+int manager_set_link_dhcp_client_iaid(const IfNameIndex *ifidx, const DHCPClient kind, const char *iaid) {
         _auto_cleanup_ char *network = NULL;
         int r;
 
@@ -385,18 +385,17 @@ int manager_set_link_dhcp_client_iaid(const IfNameIndex *ifidx, const DHCPClient
         if (r < 0)
                 return r;
 
-        r = set_config_file_int(network, kind == DHCP_CLIENT_IPV4 ? "DHCPv4" : "DHCPv6", "IAID", iaid);
+        r = set_config_file_str(network, kind == DHCP_CLIENT_IPV4 ? "DHCPv4" : "DHCPv6", "IAID", iaid);
         if (r < 0) {
                 log_warning("Failed to update DHCP IAID= to configuration file '%s': %s", network, strerror(-r));
                 return r;
         }
 
-        return 0;
+        return dbus_network_reload();
 }
 
-int manager_get_link_dhcp_client_iaid(const IfNameIndex *ifidx, const DHCPClient kind, uint32_t *iaid) {
-        _auto_cleanup_ char *network = NULL;
-        uint32_t v = 0;
+int manager_get_link_dhcp_client_iaid(const IfNameIndex *ifidx, const DHCPClient kind, char **iaid) {
+        _auto_cleanup_ char *network = NULL, *v = NULL;
         int r;
 
         assert(ifidx);
@@ -405,11 +404,14 @@ int manager_get_link_dhcp_client_iaid(const IfNameIndex *ifidx, const DHCPClient
         if (r < 0)
                 return r;
 
-        r = parse_config_file_integer(network, kind == DHCP_CLIENT_IPV4 ? "DHCPv4" : "DHCPv6", "IAID", &v);
+        r = parse_config_file(network, kind == DHCP_CLIENT_IPV4 ? "DHCPv4" : "DHCPv6", "IAID", &v);
         if (r < 0)
                 return r;
 
-        *iaid = v;
+        *iaid = strdup(v);
+        if(!*iaid)
+                return log_oom();
+
         return 0;
 }
 
@@ -445,7 +447,7 @@ int manager_set_link_dhcp_client_duid(const IfNameIndex *ifidx,
                 }
         }
 
-        return 0;
+        return dbus_network_reload();
 }
 
 int manager_set_link_mtu(const IfNameIndex *ifidx, uint32_t mtu) {
