@@ -682,6 +682,7 @@ int network_new(Network **ret) {
                 .dhcp4_send_hostname = -1,
                 .dhcp4_send_release = -1,
                 .dhcp4_use_ntp = -1,
+                .dhcp4_rapid_commit = -1,
                 .dhcp6_use_dns = -1,
                 .dhcp6_use_ntp = -1,
                 .dhcp6_use_domains = -1,
@@ -762,6 +763,8 @@ void network_free(Network *n) {
         free(n->mac);
         free(n->match_mac);
         free(n->hostname);
+        free(n->dhcp4_iaid);
+        free(n->dhcp6_iaid);
         free(n->gateway);
         free(n->dhcp4_hostname);
         free(n->req_family_for_online);
@@ -1234,12 +1237,12 @@ int generate_network_config(Network *n) {
         }
 
         if (n->dhcp4 >= 0 || n->dhcp6 >= 0) {
-               if (n->dhcp4 > 0  && n->dhcp6 > 0)
-                       r = set_config(key_file, "Network", "DHCP", "yes");
-               else if (n->dhcp4 > 0)
-                       r = set_config(key_file, "Network", "DHCP", "ipv4");
-               else if (n->dhcp6 > 0)
-                       r = set_config(key_file, "Network", "DHCP", "ipv6");
+                if (n->dhcp4 > 0  && n->dhcp6 > 0)
+                        r = set_config(key_file, "Network", "DHCP", "yes");
+                else if (n->dhcp4 > 0)
+                        r = set_config(key_file, "Network", "DHCP", "ipv4");
+                else if (n->dhcp6 > 0)
+                        r = set_config(key_file, "Network", "DHCP", "ipv6");
         }
 
         if (n->lldp >= 0) {
@@ -1406,6 +1409,30 @@ int generate_network_config(Network *n) {
                         return r;
         }
 
+        if (n->dhcp4_initial_congestion_window > 0) {
+                r = set_config_uint(key_file, "DHCPv4", "InitialCongestionWindow", n->dhcp4_initial_congestion_window);
+                if (r < 0)
+                        return r;
+        }
+
+        if (n->dhcp4_advertised_receive_window > 0) {
+                r = set_config_uint(key_file, "DHCPv4", "InitialAdvertisedReceiveWindow", n->dhcp4_advertised_receive_window);
+                if (r < 0)
+                        return r;
+        }
+
+        if (n->dhcp4_iaid) {
+                r = set_config(key_file, "DHCPv4", "IAID", n->dhcp4_iaid);
+                if (r < 0)
+                        return r;
+        }
+
+        if (n->dhcp4_rapid_commit >= 0) {
+                r = set_config(key_file, "DHCPv4", "RapidCommit", bool_to_str(n->dhcp4_rapid_commit));
+                if (r < 0)
+                        return r;
+        }
+
         /* [DHCPv6] */
         if (n->dhcp6_use_dns >= 0) {
                 r = set_config(key_file, "DHCPv6", "UseDNS", bool_to_str(n->dhcp6_use_dns));
@@ -1455,6 +1482,12 @@ int generate_network_config(Network *n) {
                         return r;
         }
 
+        if (n->dhcp6_iaid) {
+                r = set_config(key_file, "DHCPv6", "IAID", n->dhcp6_iaid);
+                if (r < 0)
+                        return r;
+        }
+
         /* [IPv6AcceptRA] */
         if (n->ipv6_ra_token) {
                 r = set_config(key_file, "IPv6AcceptRA", "Token", n->ipv6_ra_token);
